// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: PropsComposite.java
// Interfaces: PropsComposite
// Original Author: jrobbins@ics.uci.edu
// $Id: PropsComposite.java,v 1.3 1997/06/10 23:39:17 jrobbins Exp $

package uci.ui;

import java.util.*;

/** This class shows an example of how to efficiently combine
 *  IProps. Needs-More-Work: this class is not currently used in GEF,
 *  instead some classes basically duplicate this
 *  functionality. Needs-More-Work: it would be nice to implement a
 *  fly-weight pattern. */

public class PropsComposite extends Observable implements IProps {
  ////////////////////////////////////////////////////////////////
  // instance variables

  protected Vector _subs = new Vector();

  ////////////////////////////////////////////////////////////////
  // constructors

  public PropsComposite() { }

  public PropsComposite(IProps p1, IProps p2, IProps p3) {
    if (p1 != null) _subs.addElement(p1);
    if (p2 != null) _subs.addElement(p2);
    if (p3 != null) _subs.addElement(p3);
  }

  ////////////////////////////////////////////////////////////////
  // IProps API

  public Object get(String key) {
    Enumeration eachSub = _subs.elements();
    while (eachSub.hasMoreElements()) {
      IProps p = (IProps) eachSub.nextElement();
      Object res = p.get(key);
      if (res != null) return res;
    }
    return null;
  }
  public final Object get(String key, Object def) {
    Object res = get(key);
    if (null != res) return res;
    return def;
  }

  public boolean put(String key, Object value) {
    Enumeration eachSub = _subs.elements();
    while (eachSub.hasMoreElements()) {
      IProps p = (IProps) eachSub.nextElement();
      if (p.put(key, value)) return true;
    }
    return false;
  }

  public void put(Hashtable props) {
    Enumeration keys = props.keys();
    while (keys.hasMoreElements()) {
      String key = (String) keys.nextElement();
      Object value = props.get(key);
      put(key, value);
    }
  }

  public Enumeration keysIn(String category) {
    return (new Vector()).elements();
  }

  public boolean canPut(String key) {
    Enumeration eachSub = _subs.elements();
    while (eachSub.hasMoreElements()) {
      IProps p = (IProps) eachSub.nextElement();
      if (p.canPut(key)) return true;
    }
    return false;
  }

  public boolean define(String key, Object value) {
    if (null == get(key)) {
      put(key, value);
      return true;
    }
    return false;
  }

} /* end class PropsComposite */
